@@ -1378,15 +1378,21 @@ class DictTransformer(TypeTransformer[dict]):
         """
         _origin = get_origin(t)
         _args = get_args(t)
-        if _origin is not None:
-            if _origin is Annotated:
-                raise ValueError(
-                    "Flytekit does not currently have support                         for FlyteAnnotations applied to"
-                    f" dicts. {t} cannot be                         parsed."
-                )
-            if _origin is dict and _args is not None:
-                return _args
-        return None, None
+
+        if _origin is Annotated:
+            raise ValueError(
+                f"Flytekit does not currently have support \
+                    for FlyteAnnotations applied to dicts. {t} cannot be \
+                    parsed."
+            )
+        if _origin is dict and _args is not None:
+            return _args
+
+        raise ValueError(
+            "The Latch SDK does not support values with \
+                         non-generic dict typing. Use `Dict[T, T]` instead \
+                         of `dict`."
+        )
 
     @staticmethod
     def dict_to_generic_literal(v: dict) -> Literal:
