@@ -158,11 +158,21 @@ class LatchPersistence(DataPersistence):
         if not remote_path.startswith(self.PROTOCOL):
             raise ValueError(f"expected a Latch URL (latch://...): {remote_path}")
 
-        r = _session.post(
-            self._latch_endpoint + "/api/object-exists-at-url",
-            json={"object_url": remote_path, "execution_name": _os.environ.get("FLYTE_INTERNAL_EXECUTION_ID")},
-            timeout=90,
-        )
+        attempts = 0
+        while True:
+            try:
+                r = _session.post(
+                    self._latch_endpoint + "/api/object-exists-at-url",
+                    json={"object_url": remote_path, "execution_name": _os.environ.get("FLYTE_INTERNAL_EXECUTION_ID")},
+                    timeout=90,
+                )
+                break
+            except requests.exceptions.ConnectionError:
+                attempts += 1
+                if attempts >= 3:
+                    raise e
+                time.sleep(0.1 * 2**attempts)
+
         if r.status_code != 200:
             raise FlyteUserException("failed to check if object exists at url `{}`".format(remote_path))
 
@@ -202,6 +212,57 @@ class LatchPersistence(DataPersistence):
             return True
         else:
             try:
+                # Note that above, all children of the remote directory are
+                # directly placed in the destination directory, so that a remote
+                # directory that looks like
+                #
+                # latch:///source
+                # ├── a.txt
+                # ├── b.txt
+                # └── c.txt
+                #
+                # will be downloaded as
+                #
+                # /tmp/flyte-zdwdkh_k/820637e4d06dd8f3abcea74f669ab2c6
+                # ├── a.txt
+                # ├── b.txt
+                # └── c.txt
+                #
+                # instead of as
+                #
+                # /tmp/flyte-zdwdkh_k/820637e4d06dd8f3abcea74f669ab2c6
+                # └── source
+                #     ├── a.txt
+                #     ├── b.txt
+                #     └── c.txt
+                #
+                # We want that same exact behavior here, so we enforce that
+                # the source directory has a slash at the end. This leverages
+                # the fact that `latch cp` mimics GNU `cp`, which has the
+                # desired behavior if the source directory argument ends with a
+                # slash. This is shown below:
+                #
+                # $ cp -r source without_slash
+                # $ cp -r source/ with_slash
+                # $ tree .
+                # .
+                # ├── source
+                # │   ├── a.txt
+                # │   ├── b.txt
+                # │   └── c.txt
+                # ├── with_slash
+                # │   ├── a.txt
+                # │   ├── b.txt
+                # │   └── c.txt
+                # └── without_slash
+                #     └── source
+                #         ├── a.txt
+                #         ├── b.txt
+                #         └── c.txt
+
+                if not remote_path.endswith("/"):
+                    remote_path = f"{remote_path}/"
+
                 from latch_cli.services.cp import cp
 
                 cp(remote_path, local_path)
@@ -293,7 +354,7 @@ class LatchPersistence(DataPersistence):
                 raise RuntimeError("failed to complete upload for `{}`".format(to_path))
 
             data = r.json()
-            return {"cache": data["VersionId"]}
+            return {"cache": data.get("VersionId")}
         else:
             try:
                 from latch_cli.services.cp import cp
